#!/usr/bin/env python


from __future__ import division
#from cogent.core.moltype import RNA
#from cogent.parse.fasta import MinimalFastaParser
#from cogent.core.alignment import Alignment, DenseAlignment
#from cogent.core.sequence import ModelSequence, RnaSequence
#from cogent.struct.pairs_util import vienna_mask
#from cogent.struct.knots import *
from cogent.parse.fasta import FastaFinder
from cogent.struct.rna2d import ViennaStructure

#from rnalib.struct.classifier import SLBO
from rnalib.struct.pairs_util import pairs_to_mask, vienna_mask,\
    pairs_to_mask_regions
#from rnalib.struct.constraint import *
from rnalib.core.classification import Classification
from rnalib.struct.classifier import SLBO
from rnalib.struct.structgen import mfe_structs_aln, subopt_structs_aln,\
    degrade_helices, extend_helices, get_unique_pairs, keep_percentage,\
    add_bp, add_pr, mfe_constrained, subopt_constrained, mfe_structs,\
    subopt_structs
from ensemble_reg import method_C, method_H

def generate_ensemble(aln, ens_method=None, cl_method=SLBO, ref_pairs=None):

    method_id = {'C':method_C, 'H':method_H}
    if ens_method is not None and ens_method in method_id:
        method_to_use = method_id[ens_method]
        method_key_to_use = ens_method
    else:
        method_to_use = method_C
        method_key_to_use = 'C'

    ens = method_to_use(aln, pairs=ref_pairs, ref_seq=None)
    unique_pairs = get_unique_pairs(ens)
    id_pvmc = {}
    for idx, pairs in enumerate(unique_pairs):
            key = method_key_to_use + '_struct_'+str(idx)
            v, m = pairs_to_mask_regions(pairs, aln.SeqLen, recycle=True)
            assert len(v) == aln.SeqLen
            c = cl_method(pairs, aln.SeqLen)
            frac_s = c.count('S')/len(c)
            if frac_s < 0.2:
                continue
            else:
                id_pvmc[key] = {'P':pairs, 'V': v, 'M': m, 'C':c}
    return id_pvmc


def write_ensemble(ens, output_filename):
    """Store ens in output file. Return None.

    ens -- dict of {rec_id: struct_info} where struct_info is dict of
        {'P': Pairs object, 'V': ViennaStructure object,
        'M': pairing mask (str), 'C': Classification object}
    output_filename: absolute path to output file
    """
    out = open(output_filename, 'w')
    for rec_id, info in ens.items():
        out.write('>%s\n'%rec_id)
        for key in "VMC":
            out.write('%s\n'%str(info[key]))
    out.close()
    return None

def parse_ensemble(ens_filename, strict=True):
    """Return dict of {rec_id: struct_info}

    ens_filename -- absolute path to file containing the ensemble information.

    Expected file format as produced by function "write_ensemble":
    >rec_id
    vienna structure
    pairing mask
    classification string
    """
    lines = open(ens_filename)
    result = {}
    for rec in FastaFinder(lines):
        if len(rec) != 4:
            if strict:
                raise ValueError("Expecting four lines, found %s"%(len(rec)))
            else:
                continue
        else: 
            try:
                label = rec[0].strip()[1:] # strip off white space, remove >
                vienna = ViennaStructure(rec[1].strip())
                mask = rec[2].strip()
                cl = Classification(rec[3].strip())
                pairs = vienna_mask(vienna, mask)
                info = {'P':pairs, 'V':vienna, 'M':mask, 'C':cl}
                result[label] = info
            except:
                if strict:
                    raise ValueError("Cannot parse this record")
                else:
                    continue
    lines.close()
    return result



