#!/usr/bin/env python

from __future__ import division
#from ncprop import score_function, CompProp
from ncprop import *

WEIGHTS_SAME = [1,1,1,1,1]
WEIGHTS_DIFF = [1/0.674, 1/0.535, 1/0.694, 1/0.806, 1/0.719]
REF_TRAINED = [-1.270, -0.985, -0.686, -0.519, 0.814]
REF_APPROX_1 = [-2, -2, -1, -1, +2]
REF_APPROX_2 = [-1.25, -1, -0.75, -0.5, +1]
PROPS = [('STD_S_UC', STD_S_UC),('STD_S_UG', STD_S_UG),\
        ('MEAN_S_UA', MEAN_S_UA),('STD_LB_UX', STD_LBO_UX),
        ('MEAN_LBO_UA', MEAN_LBO_UA)]

def create_score_function(weights, refs, props):

    #for w_idx, weights in enumerate([weights_same, weights_diff]):
    #    for r_idx, ref in enumerate([ref_real, ref_approx, ref_approx2]):
    prop_list = []
    for (plabel, pfunc), r, w in zip(props, refs, weights):
        prop_list.append(CompProp(plabel, pfunc, RefZ=r, Weight=w))
    sf = score_function(prop_list)
    return sf

SF_UE = create_score_function(WEIGHTS_SAME, REF_APPROX_1, PROPS)
SF_UT = create_score_function(WEIGHTS_SAME, REF_TRAINED, PROPS)
SF_WE = create_score_function(WEIGHTS_DIFF, REF_APPROX_1, PROPS)
SF_WT = create_score_function(WEIGHTS_DIFF, REF_TRAINED, PROPS)

#def apply_score_function(stats, scoring_function):
#    return scoring_function(stats)

def get_scores(stats, scoring_function):
    return scoring_function(stats)

def write_scores(scores, output_filename=None):
    out = open(output_filename,'w')
    for k,v in scores.items():
        out_str = '%s\t%f\n'%(k,v)
        out.write(out_str)
    out.close()
    return None

def read_scores(lines):
    result = {}
    for line in lines:
        if not line.strip():
            continue
        label, score = line.strip().split()
        result[label] = float(score)
    return result

def get_top_scoring(ens, scores, top_cutoff=0.1, min_struct=10):
    """Return list of IDs of top-scoring structures
    """
    score_id = [(j,i) for i,j in scores.items()]
    score_id.sort() # from low to high, low=best
    num_to_select = int(top_cutoff*len(score_id))
    if num_to_select < min_struct:
        num_to_select = min_struct
    top_structs = [rec_id for (score, rec_id) in score_id[:num_to_select]]
    return top_structs


def write_top(top_scoring, output_filename=None):
    """Write top_scoring to file

    top_scoring -- list of structure IDs (string) 
    """
    out = open(output_filename, 'w')
    out.write('\n'.join(top_scoring))
    out.close()
    return None

def read_top(lines):
    """Return list of structure IDs (strings
    """
    result = []
    for line in lines:
        if not line.strip():
            continue
        result.append(line.strip())
    return result
