#!/usr/bin/env python


from __future__ import division
from cogent.core.moltype import RNA, AlphabetError
from cogent.parse.fasta import MinimalFastaParser
from cogent.core.alignment import Alignment, DenseAlignment
from cogent.core.sequence import ModelSequence
from rnalib.struct.classifier import SLBO
from rnalib.format.input import get_aln_pairs
from rnalib.struct.pairs_util import vienna_mask, pairs_to_mask

def parse_aln_fasta(lines, strict=True, dense=False,
    moltype=RNA, alphabet=None):
    """Return (Dense)Alignment object
    """
    if alphabet is None:
        alphabet = moltype.Alphabets.DegenGapped

    seqs = []
    for label, seq in MinimalFastaParser(lines):
        try:
            if dense:
                s = ModelSequence(seq, Name=label, Alphabet=alphabet)
            else:
                s = moltype.Sequence(seq, Name=label)
            seqs.append(s)
        except AlphabetError, e:
            if strict:
                raise AlphabetError(str(e))
            else:
                continue
    if dense:
        aln = DenseAlignment(seqs, MolType=moltype, Alphabet=alphabet)
    else:
        aln = Alignment(seqs, MolType=moltype)
    return aln
        

def write_aln_fasta(aln, output_filename):
    """Store aln in output file. Return None.

    aln -- (Dense)Alignment object
    output_filename: absolute path to output file
    """
    out = open(output_filename, 'w')
    out.write(str(aln))
    out.close()
    return None

def parse_aln(aln_filename, strict=True, dense=False):
    """Return dict of {rec_id: struct_info}

    aln_filename -- absolute path to file containing the ensemble information.
    strict -- boolean, if True raise Error upon encountered Error, if False,\
        skip trouble records
    dense -- boolean, if True, return DenseAlignment object, otherwise return
        Alignment object

    """
    pass

def get_aln_info(aln_filename, dense=False):
    """Return alignment information
    
    aln_filename -- path to alignment file
    dense -- if True, returns DenseAlignment object, otherwise Alignment
        object

    """
    CL_METHOD = SLBO
    # ALIGNMENT INFORMATION
    aln, cons_pairs = get_aln_pairs(open(aln_filename), dense=dense)
    cons_c = CL_METHOD(cons_pairs, aln.SeqLen)
    cons_v, cons_m = pairs_to_mask(cons_pairs, aln.SeqLen, recycle=True)

    return aln



