#!/usr/bin/env python

from __future__ import division
import pickle
from sys import argv
from numpy import average
from rnalib.util.array import unbiased_std
from rnalib.struct.classifier import SLBO
from rnalib.format.mage_util import kin_single_aln_from_cl
from rnalib.struct.structgen import *
from rnalib.format.input import get_aln_pairs, id_to_vmc
from rnalib.util.aln_util import take_sub_aln
from rnalib.format.mage_util import get_particular_value, kin_stats


def get_stats(aln, id_pvmc):

    #aln_data_filename = argv[1] # file with alignment and consensus structure
    #struct_filename = argv[2] # file with pre-generated structures
    #output = struct_filename[:-4]+'.stats' # strip off .txt

    cl_method = SLBO
    kin_axes = {0:'UC',1:'UG',2:'UA'}
    functions = [unbiased_std, average, min, max]

    #aln, pairs = get_aln_pairs(open(aln_data_filename), dense=True)
    #if len(aln.Seqs) > 100:
    #    aln = take_sub_aln(aln, size=100)
    #cons_cl = cl_method(pairs, aln.SeqLen)
    #kinemage = kin_single_aln_from_cl(aln, cons_cl)
   
    #id_vmc = id_to_vmc(open(struct_filename))
    result = {}
    for rec_id, struct_info in id_pvmc.items():
        kinemage =  kin_single_aln_from_cl(aln, struct_info['C'],\
            include_total=True)
        #if passes_fixed_rules(kinemage):
        result[rec_id] = kin_stats(kinemage)

    return result

def write_stats(stats, output_filename):
    """Write stats dictionary to file, using pickle. Return None.

    stats -- stats dict {rec_id: stats}
    output_filename -- absolute path to file where output should be written 
    """
    pickle.dump(stats, open(output_filename,'w'))
    return None

def parse_stats(stats_filename):
    """Return stats dictionary {rec_id: stats}
    
    stats_filename -- absolute path to file containing the stats
    """
    stats = pickle.load(open(stats_filename))
    return stats

def check_ens_stats(ens, stats):
    """Check same length and keys
    """
    if not len(ens) == len(stats):
        raise ValueError("Ensemble and Stats have different lengths")
    ens_keys = ens.keys()
    ens_keys.sort()
    stats_keys = stats.keys()
    stats_keys.sort()
    if ens_keys != stats_keys:
        raise ValueError("Statistics should have same keys as Ensemble")
    

