#!/usr/bin/env python
# store_stats.py

"""
Author: Sandra Smit (Sandra.Smit@colorado.edu)

Revision History:
File created on 10 May 2007.

"""
from __future__ import division
import pickle
from sys import argv
from numpy import average
from rnalib.util.array import unbiased_std
from rnalib.struct.classifier import SLBO
from rnalib.format.mage_util import kin_single_aln_from_cl
from rnalib.struct.structgen import *
from rnalib.format.input import get_aln_pairs, id_to_vmc
from rnalib.util.aln_util import take_sub_aln
from rnalib.format.mage_util import get_particular_value, kin_stats

def passes_fixed_rules(kinemage):
    """Return True when the kinemage passes all fixed rules.
    """
    k = kinemage
    sd_s_uc = get_particular_value(k, ss='S', axis='UC', func=unbiased_std)
    sd_s_ug = get_particular_value(k, ss='S', axis='UG', func=unbiased_std)
    sd_s_ua = get_particular_value(k, ss='S', axis='UA', func=unbiased_std)
    sd_t_uc = get_particular_value(k, ss='TOTAL', axis='UC', func=unbiased_std)
    sd_t_ua = get_particular_value(k, ss='TOTAL', axis='UA', func=unbiased_std)
    if sd_s_uc >= sd_t_uc:
        return False
    if sd_stem_ua < sd_t_ua:
        return False
    if sd_s_ua < sd_s_ug:
        return False
    if sd_s_ua < sd_s_uc:
        return False
    mean_s_ua = get_particular_value(k, ss='S', axis='UA', func=average)
    mean_t_ua = get_particular_value(k, ss='TOTAL', axis='UA', func=average)
    mean_l_ua = get_particular_value(k, ss='L', axis='UA', func=average)
    mean_o_ua = get_particular_value(k, ss='O', axis='UA', func=average)
    mean_b_ua = get_particular_value(k, ss='B', axis='UA', func=average)
    if mean_s_ua is not None and mean_t_ua is not None and\
        mean_s_ua >= mean_t_ua:
        return False
    if mean_s_ua is not None and mean_l_ua is not None and\
        mean_s_ua >= mean_l_ua:
        return False
    if mean_s_ua is not None and mean_o_ua is not None and\
        mean_s_ua < mean_o_ua:
        return False
    if mean_s_ua is not None and mean_b_ua is not None and\
        mean_s_ua < mean_b_ua:
        return False

    return True


if __name__ == "__main__":
    
    aln_data_filename = argv[1] # file with alignment and consensus structure
    struct_filename = argv[2] # file with pre-generated structures
    output = struct_filename[:-4]+'.stats' # strip off .txt

    cl_method = SLBO
    kin_axes = {0:'UC',1:'UG',2:'UA'}
    functions = [unbiased_std, average, min, max]

    aln, pairs = get_aln_pairs(open(aln_data_filename), dense=True)
    if len(aln.Seqs) > 100:
        aln = take_sub_aln(aln, size=100)
    #cons_cl = cl_method(pairs, aln.SeqLen)
    #kinemage = kin_single_aln_from_cl(aln, cons_cl)
   
    id_vmc = id_to_vmc(open(struct_filename))
    result = {}
    for rec_id, (v,m,c) in id_vmc.items():
        #print v
        kinemage =  kin_single_aln_from_cl(aln, c, include_total=True)
        #if passes_fixed_rules(kinemage):
        result[rec_id] = kin_stats(kinemage)
    pickle.dump(result, open(output,'w'))

