#!/usr/bin/env python

"""Basic functionality right now

"""

from __future__ import division
from cogent.struct.rna2d import ViennaStructure
from rnalib.struct.pairs_util import vienna_mask, pairs_to_mask, accuracy,\
    pairs_to_mask_regions, bp_fractions, bp_consensus
from cogent.struct.knots import PairedRegionsFromPairs, ConflictMatrix,\
    PairedRegions

def nc_cliques(paired_regions):
    #print 'NUM REGIONS', len(paired_regions)
    result = []
    bad = paired_regions
    while bad:
        good = PairedRegions()
        seen = {}
        left_over = PairedRegions()
        for pr in bad:
            if not good:
                good.append(pr)
            else:
                ok = True
                for pr2 in good:
                    if pr.conflicting(pr2):
                        ok = False
                        break
                if ok:
                    good.append(pr)
                else:
                    left_over.append(pr)
        if len(good) + len(left_over) != len(bad):
            raise ValueError("Sum not ok")
        result.append(good)
        bad = left_over
    if sum([len(i) for i in result]) != len(paired_regions):
        raise ValueError('Total sum not ok')
    #for i in result:
    #    print 'CLIQUE', len(i), i.byId().keys()
    return result

def incl_series(paired_regions):
    good = PairedRegions()
    bad = PairedRegions()
    for pr in paired_regions:
        if not good:
            good.append(pr)
        else:
            last = good[-1]
            if pr.Start > last.Start and pr.End < last.End:
                good.append(pr)
            else:
                bad.append(pr)
    return good, bad

def excl_series(paired_regions):
    good = PairedRegions()
    bad = PairedRegions()
    for pr in paired_regions:
        if not good:
            good.append(pr)
        else:
            last = good[-1]
            if pr.Start > last.End and pr.End > last.End:
                good.append(pr)
            else:
                bad.append(pr)
    return good, bad

def pairs_to_vm2(pairs, seq_len):
    symbols = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvw"
    regions = PairedRegionsFromPairs(pairs)
    #print 'INITIAL NUMBER OF REGIONS', len(regions)
    groups = nc_cliques(regions)
    if len(groups) > len(symbols):
        raise ValueError("Too many groups found")
    vienna = ['.']*seq_len
    mask = ['.']*seq_len
    for prs, mask_sym in zip(groups, symbols):
        for pr in prs:
            for x,y in pr.Pairs:
                vienna[x] = '('
                vienna[y] = ')'
                mask[x] = mask_sym
                mask[y] = mask_sym
    vienna_str = ''.join(vienna)
    mask_str = ''.join(mask)
    #print vienna_str
    #print mask_str
    return vienna_str, mask_str


def pairs_to_vm(pairs, seq_len):
    regions = PairedRegionsFromPairs(pairs)
    print 'INITIAL NUMBER OF REGIONS', len(regions)
    result = {}
    bad = regions
    curr_num = 0
    while bad:
        print 'CURR_NUM', curr_num
        nonconf = bad.nonConflicting()
        print 'NONCONF', len(nonconf)
        for pr in nonconf:
            result[pr.Id] = curr_num
        conf = bad.conflicting()
        if conf:
            print 'CONF', len(conf)
            in_good, in_bad = incl_series(conf)
            ex_good, ex_bad = excl_series(conf)
            if len(in_good) >= ex_good:
                good = in_good
                bad = in_bad
            else:
                good = ex_good
                bad = ex_bad
            print 'GOOD/BAD %s/%s'%(len(good),len(bad))
            for pr in good:
                result[pr.Id] = curr_num + 1
            curr_num += 2
        else:
            bad = []
    for k,v in result.items():
        print k,v
    nc_cliques(regions)
    """
    conf_cm = ConflictMatrix(conf)
    print conf_cm.Matrix
    #find_max_conflicts(conf_cm.Matrix.keys(), conf_cm, conf.byId())
    print conf_cm.nonConflicting()
    incl, excl = incl_series(conf)
    good, bad = excl_series(conf)
    print incl
    print good
    """
    return None


def write_struct(pairs, seq_len, output_filename):
    out = open(output_filename, 'w')
    #v, m = pairs_to_mask_regions(pairs, seq_len, recycle=True)
    v, m = pairs_to_vm2(pairs, seq_len)
    out.write("%s\n"%(str(v)))
    out.write("%s\n"%(str(m)))
    out.close()
    return None

def parse_struct(struct_filename):

    lines = open(struct_filename).readlines()
    if len(lines) != 2:
        raise ValueError("Expect two lines")
    else:
        v = ViennaStructure(lines[0].strip())
        m = lines[1].strip()
        pairs = vienna_mask(v,m)
    return pairs
   

def get_bp_consensus(ens, scores, top_cutoff=0.1, bp_cutoff=0.5, min_struct=10):
    score_id = [(j,i) for i,j in scores.items()]
    score_id.sort() # from low to high, low=best
    num_to_select = int(top_cutoff*len(score_id))
    if num_to_select < min_struct:
        num_to_select = min_struct
    top_structs = [ens[rec_id]['P'] for (score, rec_id) in\
                    score_id[:num_to_select]]
    bp_fracs = bp_fractions(top_structs)
    bp_cons = bp_consensus(bp_fracs, cutoff=bp_cutoff)
    return bp_cons
