#!/usr/bin/env python
"""
Functions should return figure object. If filename provided, should save fig.
"""
from __future__ import division
from numpy import array, arange
#lazy import
from pylab import *
#from pylab import clf, scatter, axvline, text, grid, xlim, xlabel,ylabel,\
#    savefig
from matplotlib.font_manager import FontProperties
from cogent.maths.stats.test import correlation

def plot_rank_vs_score(scores, output_filename=None):
    """Show or save XY Plot of rank (x-axis) versus score (y-axis)
    
    scores -- dict of {rec_id: RMSD_score}

    """
    this_fig = figure()
    this_fig_fontsize = 14
    this_fig_textsize = 10
    clf()
    score_vals = array(scores.values())
    score_vals.sort() # low to high scores
    five_idx = round(0.05*len(score_vals))-1
    ten_idx = round(0.1*len(score_vals))-1
    tf_idx = round(0.25*len(score_vals))-1
    rank_vals = arange(len(score_vals))
    scatter(rank_vals, score_vals, c='#000000')
    max_val = score_vals[-1]
    shift = 0.005*len(score_vals)
    axvline(five_idx, color='yellow')
    axvline(ten_idx, color='orange')
    axvline(tf_idx, color='red')
    text(five_idx+shift, max_val-(0.15*max_val), '5%',\
        fontsize=this_fig_textsize)
    text(ten_idx+shift, max_val-(0.1*max_val), '10%',\
        fontsize=this_fig_textsize)
    text(tf_idx+shift, max_val-(0.05*max_val), '25%',\
        fontsize=this_fig_textsize)
    grid()
    xlim(xmin=0)
    xlabel('Rank', fontsize=this_fig_fontsize)
    ylabel('RMSD score', fontsize=this_fig_fontsize)
    if output_filename is not None:
        savefig(output_filename)
    return this_fig
    

def plot_ens_acc(ens_acc, ens_cl_acc, output_filename=None):
    """Plot accuracy values in ensemble (SEN, PPV, CC, CS)
    """
    this_fig_w = 4 #width
    this_fig_h = (3/2)*this_fig_w #height
    subplot_w = 0.28
    subplot_h = (this_fig_w/this_fig_h)*subplot_w
    left_m = 0.12
    bottom_m = (this_fig_w/this_fig_h)*left_m
    this_fs = 10 #fontsize

    this_fig = figure(figsize=(this_fig_w,this_fig_h))
    clf()
    
    sen_vals = []
    ppv_vals = []
    cc_vals = []
    cs_vals = []
    
    for struct_id in ens_acc.keys():
        cs_vals.append(ens_cl_acc[struct_id])
        cc_vals.append(ens_acc[struct_id]['CC'])
        ppv_vals.append(ens_acc[struct_id]['PPV'])
        sen_vals.append(ens_acc[struct_id]['SEN'])

    sen_avg = array(sen_vals).mean() 
    #sen_std = unbiased_std(array(sen_vals))
    sen_std = array(sen_vals).std()
    ppv_avg = array(ppv_vals).mean()
    ppv_std = array(ppv_vals).std()
    cc_avg = array(cc_vals).mean()
    cc_std = array(cc_vals).std()
    cs_avg = array(cs_vals).mean()
    cs_std = array(cs_vals).std()

    figtext(left_m, 0.90,\
        u'%s structures\n'%(len(ens_acc))+\
        u'SEN: %.3f\u00b1%.3f\nCC: %.3f\u00b1%.3f'\
        %(sen_avg, sen_std, cc_avg, cc_std), size=this_fs)
    figtext(0.5+left_m, 0.9, u'PPV: %.3f\u00b1%.3f\nCS: %.3f\u00b1%.3f'\
        %(ppv_avg, ppv_std, cs_avg, cs_std), size=this_fs)

    sen_ax = axes([0+left_m, .6+bottom_m,subplot_w,subplot_h],\
        axisbg='#FFFFFF')
    #sen_ax = subplot(3,2,1)
    num_bins = (max(sen_vals)-min(sen_vals))/0.05
    if num_bins < 3:
        num_bins = 3
    n, bins, patches = hist(sen_vals, num_bins, normed=0,\
        fc='#000000', ec='#CCCCCC')
    axvline(x=sen_avg, c='#666666', lw=1.5, ls='--')
    xlabel('SEN', size=this_fs)
    setp(sen_ax, xlim=(0,1), xticks=[0,1], yticks=[])
    
    ppv_ax = axes([1/2+left_m, .6+bottom_m,subplot_w,subplot_h],\
        axisbg='#FFFFFF')
    #ppv_ax = subplot(3,2,2)
    num_bins = (max(ppv_vals)-min(ppv_vals))/0.05
    if num_bins < 3:
        num_bins = 3
    n, bins, patches = hist(ppv_vals, num_bins, normed=0,
        fc='#000000', ec='#CCCCCC')
    axvline(x=ppv_avg, c='#666666', lw=1.5, ls='--')
    xlabel('PPV', size=this_fs)
    setp(ppv_ax, xlim=(0,1), xticks=[0,1], yticks=[])


    cc_ax = axes([0+left_m,.3+bottom_m,subplot_w,subplot_h],\
        axisbg='#FFFFFF')
    #cc_ax = subplot(3,2,3)
    num_bins = (max(cc_vals)-min(cc_vals))/0.05
    if num_bins < 3:
        num_bins = 3
    n, bins, patches = hist(cc_vals, num_bins, normed=0,
        fc='#000000', ec='#CCCCCC')
    axvline(x=cc_avg, c='#666666', lw=1.5, ls='--')
    xlabel('CC', size=this_fs)
    setp(cc_ax, xlim=(0,1), xticks=[0,1], yticks=[])

    cs_ax = axes([1/2+left_m,.3+bottom_m,subplot_w,subplot_h],
        axisbg='#FFFFFF')
    #cs_ax = subplot(3,2,4)
    num_bins = (max(cs_vals)-min(cs_vals))/0.05
    if num_bins < 3:
        num_bins = 3
    n, bins, patches = hist(cs_vals, num_bins, normed=0,
        fc='#000000', ec='#CCCCCC')
    axvline(x=cs_avg, c='#666666', lw=1.5, ls='--')
    xlabel('CS', size=this_fs)
    setp(cs_ax, xlim=(0,1), xticks=[0,1], yticks=[])

    sen_ppv_ax = axes([0+left_m,0+bottom_m,subplot_w,subplot_h],\
        axisbg='#CCCCCC')
    #sen_ppv_ax = subplot(3,2,5)
    scatter(sen_vals, ppv_vals, s=20, c='#000000', edgecolor='#333333')
    xlabel('SEN', size=this_fs)
    ylabel('PPV', size=this_fs)
    setp(sen_ppv_ax, xlim=(0,1), ylim=(0,1), xticks=[0,1], yticks=[0,1])

    cc_cs_ax = axes([1/2+left_m,0+bottom_m,subplot_w,subplot_h],\
        axisbg='#CCCCCC')
    #cc_cs_ax = subplot(3,2,6)
    scatter(cc_vals, cs_vals, s=20, c='#000000', edgecolor='#333333')
    xlabel('CC', size=this_fs)
    ylabel('CS', size=this_fs)
    setp(cc_cs_ax, xlim=(0,1), ylim=(0,1), xticks=[0,1], yticks=[0,1])

    if output_filename is not None:
        savefig(output_filename)
    return this_fig

def hist_values(values, start=0, end=1, step=0.05):
    val = array(values)
    bins = frange(start, end, step)[:-1]
    s = searchsorted(bins, val)
    items = []
    for idx, start_val in enumerate(bins):
        items.append((idx, start_val, len((s==idx+1).nonzero()[0])/len(val)))
    return items


def plot_ens_top(id_acc, top_ids, enscon_acc=None, topcon_acc=None,\
    acc_metric='CC', output_filename=None):
    """Plot accuracy of ens and top + averages and consensus

    id_acc -- dict of {struct_id: accuracy}
    top_ids -- list of ids in top-scoring set

    WARNING: two lines might overlap at the moment and then one
    of them is not visible.
    """
    top_ids = dict.fromkeys(top_ids)
    ens_cc_vals = array([v[acc_metric] for rec_id, v in id_acc.items()])
    ens_hist = hist_values(ens_cc_vals, 0,1,0.05)
    top_cc_vals = array([v[acc_metric] for rec_id, v in id_acc.items()\
        if rec_id in top_ids])
    top_hist = hist_values(top_cc_vals, 0,1,0.05)

    this_fig_fontsize = 10
    this_fig = figure()
    clf()
    axes([.12,.1,.8,.75])
    b_fake = bar([-0.00001],[0], width=0, color='white')
    b_fake = bar([-1.00001],[0], width=0, color='white')
    b1 = bar(array([i[1] for i in ens_hist]),\
        [i[2] for i in ens_hist],\
        width=0.045, color='black',alpha=1.0)
    axvline(ens_cc_vals.mean(), c='k', ls='--', lw=2)
    if enscon_acc:
        axvline(enscon_acc[acc_metric], c='k', ls='-',lw=2)
    b2 = bar(array([i[1] for i in top_hist])+0.005,\
        [i[2] for i in top_hist],\
        width=0.035, color='gray',alpha=0.8)
    axvline(top_cc_vals.mean(), c='gray', ls='--', lw=2)
    if topcon_acc:
        axvline(topcon_acc[acc_metric], c='gray', ls='-',lw=2)
    xlim((-0.05, 1.05))
    xlabel("Accuracy (%s)"%(acc_metric), fontsize=this_fig_fontsize+2)
    ylabel("Fraction of structures", fontsize=this_fig_fontsize+2)
    y_of_ticks = gca().get_ylim()[1]
    y_of_text = y_of_ticks+0.03*y_of_ticks
    text(ens_cc_vals.mean(), y_of_text, "mean",\
        fontsize=this_fig_fontsize, rotation=45, horizontalalignment='left')
    text(top_cc_vals.mean(), y_of_text, "mean",\
        fontsize=this_fig_fontsize, rotation=45, horizontalalignment='left')
    if enscon_acc is not None:
        text(enscon_acc[acc_metric], y_of_text, "consensus",\
            fontsize=this_fig_fontsize, rotation=45, horizontalalignment='left')
    if topcon_acc is not None:
        text(topcon_acc[acc_metric], y_of_text, "consensus",\
            fontsize=this_fig_fontsize, rotation=45, horizontalalignment='left')
    legend((b1[0],b2[0]), ('\nAll structures (%s)'%(len(ens_cc_vals)),\
        '\nTop-scoring structures (%s)'%len(top_cc_vals)),\
        loc='upper left', prop=FontProperties(size=this_fig_fontsize),\
        pad=0.15, handlelen=0.02)

    if output_filename is not None:
        savefig(output_filename)
    return this_fig

def plot_score_acc(id_acc, scores, acc_metric='CC', output_filename=None):
    """Plots accuracy (y-axis) versus score (x-axis)
    """

    this_fig_fontsize = 10
    this_fig = figure()
    clf()

    acc_vals = []
    score_vals = []
    for struct_id in id_acc:
        acc_vals.append(id_acc[struct_id][acc_metric])
        score_vals.append(scores[struct_id])
    r_val, p_val = correlation(acc_vals, score_vals)
    r2_val = r_val ** 2
    
    scatter(score_vals, acc_vals, s=20, c='#000000', edgecolor='#333333')
    ylim((0,1))
    xlim(xmin=0)
    xlabel('RMSD Score', fontsize=this_fig_fontsize+2)
    ylabel('Accuracy (%s)'%acc_metric, fontsize=this_fig_fontsize+2)

    x_max = gca().get_xlim()[1]
    y_max = gca().get_ylim()[1]
    corr_text = u"Correlation\nr = %.3f\nr\u00b2 = %.3f\nP = %.3e"\
        %(r_val, r2_val,p_val)
    text(0.8*x_max, 0.85*y_max, corr_text, fontsize=this_fig_fontsize)
    if output_filename is not None:
        savefig(output_filename)
    return this_fig
